// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //

#include <cassert>
#include <iostream>
#include <fstream>
#include <string>
#include <complex>
#include <cstdlib>

#include "3bamp.h"

#include <gsl/gsl_math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_sf_coupling.h>
#include <gsl/gsl_sf_legendre.h>

#include <gsl/gsl_multifit.h>




//double sgfitup = 0.160;
//double sgfitlow = 0.104;



//int fitdim = 6;

double grefitparam[8][8][8][8][10] ;
double gimfitparam[8][8][8][8][10] ;



// isospin recoupling matrix //
const double Isobeta[3][3] = {{1.0/3.0,1.0,5.0/3.0},
                              {1.0/3.0,1.0/2.0,-5.0/6.0},
                              {1.0/3.0,-1.0/2.0,1.0/6.0}};
// ------------------------- //




double svec_struct[4100];
double GLvecre_struct[8][8][8][8][4100];
double GLvecim_struct[8][8][8][8][4100];


// amplitudes in terms of (x,y) //


complex<double>	ThreebodyAmp::amp_xy_3b(double x, double y, int m1, int m2, int m3,int Imax, int Lmax,  double meta, double mpip, double mpim, double mpi0, struct param3b_space *pa0_IL)
{
    double Qeta = meta - mpip - mpim - mpi0;
    
    double metasq = meta*meta;
    double mpipsq = mpip*mpip;
    double mpimsq = mpim*mpim;
    double mpi0sq = mpi0*mpi0;
    
    
    
    double s = pow(meta-mpi0,2) - 2.0*meta*Qeta/3.0*(y+1.0);
    double t = meta*Qeta/sqrt(3.0)*x + (metasq+mpipsq+mpimsq+mpi0sq-s)/2.0;
    double u = metasq+mpipsq+mpimsq+mpi0sq-s-t;
    
    double phi = s*t*u - u*(mpi0sq*mpipsq + metasq*mpimsq)
    - s*(mpipsq*mpimsq + metasq*mpi0sq)
    - t*(mpimsq*mpi0sq + metasq*mpipsq)
    + 2.0*mpi0sq*mpipsq*mpimsq + 2.0*metasq*(mpipsq*mpimsq + mpipsq*mpi0sq + mpi0sq*mpimsq);
    
    
    
    complex<double>	 r (0.0,0.0);
    
    if (phi>=0.0)
    {
        r = allamp3b(s,t,m1,m2,m3,Imax,Lmax, meta,mpip,mpim,mpi0,pa0_IL);
    }
    
    return r;
}





complex<double>	ThreebodyAmp::allamp3b(double s, double t, int m1, int m2, int m3,int Imax, int Lmax,double meta, double mpip, double mpim, double mpi0, struct param3b_space *pa0_IL)
{
    complex<double>	 r (0.0,0.0);
    
    for (int i=1; i<=paramfitnum3b; i++)
    {
        int Ip = a0fit3b_I[i];
        int Lp = a0fit3b_L[i];
        
        r = r + (*pa0_IL).a0_I_L_struct[Ip][Lp]*totamp3b(s,t,m1,m2,m3,Imax,Lmax,Ip,Lp, meta,mpip,mpim,mpi0);
    }
    
    return r;
}








complex<double>	ThreebodyAmp::totamp3b(double s, double t,  int m1, int m2, int m3,int Imax, int Lmax, int Ip, int Lp, double meta, double mpip, double mpim, double mpi0) const
{
    complex<double>	 r (0.0,0.0);
    
    if (int(1.0+pow(-1.0,Ip+Lp))!=0)
    {
        double metasq = meta*meta;
        double mpipsq = mpip*mpip;
        double mpimsq = mpim*mpim;
        double mpi0sq = mpi0*mpi0;
        
        double u = metasq + mpipsq + mpimsq + mpi0sq - s - t;
        double phi = s*t*u - u*(mpi0sq*mpipsq + metasq*mpimsq)
        - s*(mpipsq*mpimsq + metasq*mpi0sq)
        - t*(mpimsq*mpi0sq + metasq*mpipsq)
        + 2.0*mpi0sq*mpipsq*mpimsq + 2.0*metasq*(mpipsq*mpimsq + mpipsq*mpi0sq + mpi0sq*mpimsq);
        
        
        if (phi>=0.0)
        {
            
            double cos_s = - ( 2.0*s*( metasq + mpipsq - t ) - (s + mpipsq - mpimsq)*(metasq + s - mpi0sq) )/
            ( sqrt( (s-pow(mpip-mpim,2))*(s-pow(mpip+mpim,2)))*sqrt((s-pow(meta-mpi0,2))*(s-pow(meta+mpi0,2)) ) );
            
            double cos_t = ( 2.0*t*( metasq + mpimsq - u ) - (t + mpimsq - mpi0sq)*(metasq + t -mpipsq) )/
            ( sqrt( (t-pow(mpim-mpi0,2))*(t-pow(mpim+mpi0,2)))*sqrt((t-pow(meta-mpip,2))*(t-pow(meta+mpip,2)) ) );
            
            double cos_u =  ( 2.0*u*( metasq + mpi0sq - s ) - (u + mpi0sq - mpipsq)*(metasq +u-mpimsq) )/
            ( sqrt( (u-pow(mpi0-mpip,2))*(u-pow(mpi0+mpip,2)))*sqrt((u-pow(meta-mpim,2))*(u-pow(meta+mpim,2)) ) );
            
            for (int Is=0; Is<=Imax; Is++)
            {
                for (int L=0; L<=Lmax; L++)
                {
                    if (int(1.0+pow(-1.0,Is+L))!=0)
                    {
                        r = r + pow(-1.0,Is)*(2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_s)*amp_PW_3b(s,m1,m2,m3,Is,L,Ip,Lp, meta,mpip,mpim,mpi0)
                        + (2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_t)*amp_PW_3b(t,m2,m3,m1,Is,L,Ip,Lp, meta,mpim,mpi0,mpip)
                        + (2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_u)*amp_PW_3b(u,m3,m1,m2,Is,L,Ip,Lp, meta,mpi0,mpip,mpim);
                        
                    }
                }
            }
        }
    }
    
    
    return r;
}








complex<double>	ThreebodyAmp::amp_PW_3b(double s,int m1, int m2, int m3, int Is, int L, int Ip, int Lp,double meta, double mass1, double mass2, double mass3) const
{
    complex<double>	 r (0.0,0.0);
    
    if (int(1.0+pow(-1.0,Is+L))!=0)
    {
        r = IS_coef(Is,m1,m2,m3)*pow(s*sqrt(Ksq(s,meta,mass1,mass2,mass3)),L)*T3bhatIL(s,Is,L,Ip,Lp, meta); // new barrier //
    }
    
    return r;
}







complex<double>	ThreebodyAmp::T3bhatIL(double s,int I, int L, int Ip, int Lp, double meta) const
{
    complex<double> r (0.0,0.0);
    
    int ncut = 2;
    double slow = svec_struct[1];
    double sup  = pow(meta-0.13498,2);

    
    if ((s>slow)&&(s<sup))
    //if (s>slow)
    {

        double eps = 0.1e-30;
        double gre = enatk(svec_struct,GLvecre_struct[Ip][Lp][I][L],ndimright-ncut, s, eps);
        double gim = enatk(svec_struct,GLvecim_struct[Ip][Lp][I][L],ndimright-ncut, s, eps);
        
        /*
        gsl_interp_accel *acc = gsl_interp_accel_alloc ();
        
        gsl_spline *spline_re = gsl_spline_alloc (gsl_interp_cspline, ndimright-ncut);
        gsl_spline *spline_im = gsl_spline_alloc (gsl_interp_cspline, ndimright-ncut);
        
        gsl_spline_init (spline_re, svec_struct, GLvecre_struct[Ip][Lp][I][L], ndimright-ncut);
        gsl_spline_init (spline_im, svec_struct, GLvecim_struct[Ip][Lp][I][L], ndimright-ncut);
        
        double gre = gsl_spline_eval (spline_re, s, acc);
        double gim = gsl_spline_eval (spline_im, s, acc);
        
        gsl_spline_free (spline_re);
        gsl_spline_free (spline_im);
        gsl_interp_accel_free (acc);
        */
        
        complex<double> gIL (gre,gim);
        
        //if ((s<sgfitup)&&(s>sgfitlow))
        
        
        if (s>sgfitlow)
        {
            gIL = GILfit(Ip,Lp,I,L,s);
        }
        
        
        complex<double> that = thatIL(s,I,L);
        
        
        //r = that*(gre + aIm*gim);
        r = that*gIL;
    }
    
    return r;
}


// ------------------------- //














// three-body rescattering //



void	ThreebodyAmp::GhatLinv(int Imax, int Lmax, int It, int Lt, double meta) const
{
    
    int nmatsize = 0;
    int store_I[20];
    int store_L[20];
    
    int *pstore_I = store_I;
    int *pstore_L = store_L;

   
    // determine the size of matrix //
    int m1=1;
    int m2=-1;
    int m3=0;
    
    for (int Is=0; Is<=Imax; Is++)
    {
        for (int Ls=0; Ls<=Lmax; Ls++)
        {
            if ((int(1.0+pow(-1.0,Is+Ls))!=0)&&((IS_coef(Is,m1,m2,m3)!=0.0)||(IS_coef(Is,m2,m3,m1)!=0.0)||(IS_coef(Is,m3,m1,m2)!=0.0)))
            {
                nmatsize = nmatsize + 1;
                pstore_I[nmatsize] = Is;
                pstore_L[nmatsize] = Ls;
            }
        }
    }
    // ---------------------------- //

    
    
    // matrix inversion //
    int n = paramfitnum3b*ndimright;
    complex<double>   *invmat  = new complex<double>[(n+1)*(n+1)];
	
	invertmatrix(paramfitnum3b,pstore_I, pstore_L, n,invmat,meta );
    // --------------- //
    
    
    
    
    double gini[20][20];
    

    // determine the n-th solution //
    int nsol = 0;
    for (int i=1; i<=paramfitnum3b; i++)
    {
        if ((pstore_I[i]==It)&&(pstore_L[i]==Lt))
        {
            nsol = i;
        }
    }

    for (int i=1; i<=nmatsize; i++)
    {
        if (i==nsol)
        {
            gini[nsol][i] = 1.0;
        }
        else
        {
            gini[nsol][i] = 0.0;
        }
    }
    
    
 
    
    
    
    // getting the solution of G //
    
    int Ip = pstore_I[nsol];
    int Lp = pstore_L[nsol];
    
    
    
    double GLre[Imax+1][Lmax+1][4100];
    double GLim[Imax+1][Lmax+1][4100];
    
    
    complex<double>   gres[20][4100];
    
    for (int nmat=1; nmat<=nmatsize; nmat++)
    {
        for (int i=1; i<=ndimright; i++)
        {
            for (int mmat=1; mmat<=nmatsize; mmat++)
            {
                for (int j=1; j<=ndimright; j++)
                {
                    int ii = i + (nmat-1)*ndimright;
                    int jj = j + (mmat-1)*ndimright;
                    int k = (ii-1)*nmatsize*ndimright + (jj-1);
                    
                    gres[nmat][i] = gres[nmat][i] + invmat[k]*gini[nsol][mmat];
                }
            }
        }
    }
    
    
    
    
    for (int nmat=1; nmat<=nmatsize; nmat++)
    {
        for (int i=1; i<=ndimright; i++)
        {
            GLre[pstore_I[nmat]][pstore_L[nmat]][i] = real(gres[nmat][i]);
            GLim[pstore_I[nmat]][pstore_L[nmat]][i] = imag(gres[nmat][i]);
        }
    }
    
    
    
    
    
    for (int I1=0; I1<=Imax; I1++)
    {
        for (int L1=0; L1<=Lmax; L1++)
        {
            if (int(1.0+pow(-1.0,I1+L1))!=0)
            {
                for (int i=1; i<=ndimright; i++)
                {
                    double s = xradright[i];
                    svec_struct[i] = s ;
                    
                    GLvecre_struct[Ip][Lp][I1][L1][i] = GLre[I1][L1][i];
                    GLvecim_struct[Ip][Lp][I1][L1][i] = GLim[I1][L1][i];
                }
            }
        }
    }
    
 
    delete invmat;
}





// Inverse the matrix.

void ThreebodyAmp::invertmatrix(int nmatsize, int *pstore_I, int *pstore_L, int n, complex<double> *aaa , double meta) const
{
	
    
    for (int nmat=1; nmat<=nmatsize; nmat++)
    {
        for (int i=1; i<=ndimright; i++)
        {
            for (int mmat=1; mmat<=nmatsize; mmat++)
            {
                for (int j=1; j<=ndimright; j++)
                {
                    int ii = i + (nmat-1)*ndimright;
                    int jj = j + (mmat-1)*ndimright;
                    
                    int k = (ii-1)*n+(jj-1);
                    aaa[k] = mat(nmat,i,mmat,j, pstore_I, pstore_L, meta);
                }
            }
        }
    }
    
    
	
	int ipv[n];
	int LDA = n;
	
	complex <double> det[2];
	int job = 01;
	
	zgeco(aaa,LDA,n,ipv);
	zgedi(aaa,LDA,n,ipv,det,job);
	
	
}








complex<double>  ThreebodyAmp::mat(int nmat, int i, int mmat, int j, int *pstore_I, int *pstore_L, double meta) const
{
    
	double adel;
	if ((i==j)&&(nmat==mmat))
	{
		adel = 1.0;
	}
	else
	{
		adel = 0.0;
	}
	
    
	complex<double> aij = mat_I_L_ij(pstore_I[nmat], pstore_L[nmat], pstore_I[mmat], pstore_L[mmat],i,j,meta);
    
	
	complex<double> matij     =  adel - aij;
	
	return matij;
}


 

complex<double>  ThreebodyAmp::mat_I_L_ij(int Is, int Ls, int It, int Lt,int i, int j, double meta) const
{
	double si  =  xradright[i];
	double sj  =  xradright[j];
	
    complex<double> at      = Isobeta[Is][It]*thatIL(sj,It,Lt)/pow(4.0,Ls)*(AdlerZeroDeltaL1L2(Ls,Lt,si,sj,meta,Is)-AdlerZeroDeltaL1L2(Ls,Lt,s3bsub,sj,meta,Is)); // Adler Zero //
	
	complex<double> aij     =   2.0*weradright[j]*at;
	
	return aij;
}



// ------------------------------------- //




























// rescattering kernels //



complex<double>	ThreebodyAmp::Delta0t(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> Lc     =  - log( pow( (meta*meta-mpi*mpi+tc) + sqk,2)/(4.0*meta*meta*tc) );
        
        complex<double> delta  ( real(Lc), 0.0);
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}






complex<double>	ThreebodyAmp::Delta1t(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> Lc     =  sqk/(meta*(meta-mpi)*(tc-mpi*(meta+mpi)));
        
        complex<double> delta  ( real(Lc), 0.0);
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::Delta2t(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> Lc     =  sqk/(meta*(meta+mpi)*(tc+mpi*(meta-mpi)));
        
        complex<double> delta  ( real(Lc), 0.0);
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}




// ------------ //



complex<double>	ThreebodyAmp::Delta3st(double s, double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> delta     =  sqk/PI + (meta*meta+mpi*mpi-s)*Delta0t(t,meta);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta3tplus(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> delta     =  sqk/PI - 2.0*meta*mpi*Delta0t(t,meta);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}


complex<double>	ThreebodyAmp::Delta3tminus(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        
        complex<double> delta     =  sqk/PI + 2.0*meta*mpi*Delta0t(t,meta);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}









complex<double>	ThreebodyAmp::Delta02tplus(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        
        complex<double> delta   = 6.0*Delta3tplus(t,meta)
        + 6.0*(meta*meta+4.0*meta*mpi-mpi*mpi+t)*Delta0t(t,meta)
        + (3.0*pow(meta*meta+4.0*meta*mpi-mpi*mpi+t,2)-t*(t-4.0*mpi*mpi)*(t-sbplus)*(t-sbminus)/t/t)/2.0*Delta2t(t,meta);
        
        return 5.0*delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta02tminus(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        
        complex<double> delta   = 6.0*Delta3tminus(t,meta)
        + 6.0*(meta*meta-4.0*meta*mpi-mpi*mpi+t)*Delta0t(t,meta)
        + (3.0*pow(meta*meta-4.0*meta*mpi-mpi*mpi+t,2)-t*(t-4.0*mpi*mpi)*(t-sbplus)*(t-sbminus)/t/t)/2.0*Delta1t(t,meta);
        
        return 5.0*delta;
    }
	else
    {
        return 0.0;
    }
}






double ThreebodyAmp::Phi_st(double s, double t, double meta) const
{
    double phi = s*t*(meta*meta+3.0*mpi*mpi-s-t) - mpi*mpi*pow(meta*meta-mpi*mpi,2);
    return phi;
}





complex<double>	ThreebodyAmp::Delta4t(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        double phi = Phi_st(sbminus,t,meta);
        
        complex<double> Lc     =  sqk/phi*(3.0+(mpi*mpi*sqk*sqk+3.0*meta*meta*t*(t-4.0*mpi*mpi))/phi);
        Lc = - 4.0/3.0/(16.0*meta*meta*mpi*mpi)*mpi*(meta-mpi)*(t-mpi*(meta+mpi))*Lc;
        
        complex<double> delta  ( real(Lc), 0.0);
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}





complex<double>	ThreebodyAmp::Delta5t(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double eps = 1.0E-30;
        complex<double>	tc        (t,eps);
        complex<double>	sqk    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        double phi = Phi_st(sbplus,t,meta);
        
        complex<double> Lc     =  sqk/phi*(3.0+(mpi*mpi*sqk*sqk+3.0*meta*meta*t*(t-4.0*mpi*mpi))/phi);
        Lc =  4.0/3.0/(16.0*meta*meta*mpi*mpi)*mpi*(meta+mpi)*(t+mpi*(meta-mpi))*Lc;
        
        complex<double> delta  ( real(Lc), 0.0);
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}






// ------------ //



complex<double>	ThreebodyAmp::Delta00(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if (t<pow(meta-mpi,2))
    {
        double eps = 1.0E-30;
        complex<double>	sc        (s,eps);
        complex<double>	tc        (t,eps);
        double          R       =   - pow(meta,4) + meta*meta*(s+t) + (s-mpi*mpi)*(t-mpi*mpi);
        complex<double>	sqk1    =  cpsqrt( (sc-sbplus)*(sc-sbminus) );
        complex<double>	sqk2    =  cpsqrt( (tc-sbplus)*(tc-sbminus) );
        complex<double> Uc      =  sqk1*sqk2;
        
        complex<double> Lc      =  log(abs((R+Uc)/(R-Uc)));
        
        
        
        
        //complex<double> aIm (0.0,1.0);
        
        complex<double> delta      (real(Lc/sqk1),0.0);
        
        // decay region //
        if (t>4.0*mpi*mpi)
        {
            if ((s>real(Sminus(t,meta)))&&(s<real(Splus(t,meta))))
            {
                delta = delta - aIm*PI/sqk1;
            }
            
            if ((s>pow(meta-mpi,2))&&(s<pow(meta+mpi,2)))
            {
                delta =  log((R+Uc)/(R-Uc))/sqk1;
                
                if (t<mpi*(meta+mpi))
                {
                    double sct = (meta*meta*(meta*meta-t)-mpi*mpi*(mpi*mpi-t))/(meta*meta-mpi*mpi+t);
                    if (s<sct)
                    {
                        delta = delta - 2.0*aIm*PI/sqk1;
                    }
                }
            }
        }
        
        
        // unphysical region //
        if ((t<4.0*mpi*mpi)&&(t>0.0))
        {
            if ((s>pow(meta-mpi,2))&&(s<pow(meta+mpi,2)))
            {
                delta = log((R+Uc)/(R-Uc))/sqk1;
                
                double sct = (meta*meta*(meta*meta-t)-mpi*mpi*(mpi*mpi-t))/(meta*meta-mpi*mpi+t);
                if (s<sct)
                {
                    delta = delta - 2.0*aIm*PI/sqk1;
                }
            }
        }
        
        
        // scattering region //
        if (t<0.0)
        {
            if (s>real(Splus(t,meta)))
            {
                delta = delta + aIm*PI/sqk1;
            }
            
            if (s<real(Sminus(t,meta)))
            {
                delta = delta + aIm*PI/sqk1;
            }
            
            if ((s>pow(meta-mpi,2))&&(s<pow(meta+mpi,2)))
            {
                delta = log((R+Uc)/(R-Uc))/sqk1;
                
                double sct = (meta*meta*(meta*meta-t)-mpi*mpi*(mpi*mpi-t))/(meta*meta-mpi*mpi+t);
                if (s<sct)
                {
                    delta = delta - 2.0*aIm*PI/sqk1;
                }
            }
        }
        
        
        return delta/PI;
    }
	else
    {
        return 0.0;
    }
}













complex<double>	ThreebodyAmp::Delta01(double s,double t, double meta) const
{
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = 6.0*Delta0t(t,meta) + 3.0*(2.0*s+t-meta*meta-3.0*mpi*mpi)*Delta00(s,t,meta);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}





complex<double>	ThreebodyAmp::Delta10(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( 1.0+(2.0*t+2.0*mpi*(meta-mpi))/(s-sbplus)+sbminus/(s-sbminus)
                                    +sbminus*(2.0*t+2.0*mpi*(meta-mpi))/(s-sbminus)/(s-sbplus) )*Delta00(s,t,meta)
        +  sbminus*(2.0*t-2.0*mpi*(meta+mpi))/(s-sbminus)/(4.0*meta*mpi)*Delta1t(t,meta)
        -  sbplus*(2.0*t+2.0*mpi*(meta-mpi))/(s-sbplus)/(4.0*meta*mpi)*Delta2t(t,meta) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta11(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = 6.0*Delta0t(t,meta) + 3.0*( 2.0*s+t-meta*meta-3.0*mpi*mpi )*Delta10(s,t,meta)
        -  6.0*sbminus*(2.0*t-2.0*mpi*(meta+mpi))/(4.0*meta*mpi)*Delta1t(t,meta)
        +  6.0*sbplus*(2.0*t+2.0*mpi*(meta-mpi))/(4.0*meta*mpi)*Delta2t(t,meta) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



// -------------------- //




complex<double>	ThreebodyAmp::Delta02(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = 6.0*Delta3st(s,t,meta)
        + 6.0*(2.0*s+t-meta*meta-3.0*mpi*mpi)*Delta0t(t,meta)
        + (3.0*pow(2.0*s+t-meta*meta-3.0*mpi*mpi,2)-t*(t-4.0*mpi*mpi)*(t-sbminus)*(t-sbplus)/t/t)/2.0*Delta00(s,t,meta)  ;
        
        return 5.0*delta;
    }
	else
    {
        return 0.0;
    }
}






complex<double>	ThreebodyAmp::Delta12(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = ( 1.0+(2.0*t+2.0*mpi*(meta-mpi))/(s-sbplus) + pow(meta-mpi,2)/(s-sbminus) + pow(meta-mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))/(s-sbminus)/(s-sbplus) )*Delta02(s,t,meta)
        - 1.0/(4.0*meta*mpi)*pow(meta+mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))/(s-sbplus)*Delta02tplus(t,meta)
        + 1.0/(4.0*meta*mpi)*pow(meta-mpi,2)*(2.0*t-2.0*mpi*(meta+mpi))/(s-sbminus)*Delta02tminus(t,meta)  ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}








complex<double>	ThreebodyAmp::Delta00A(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( Delta00(s,t,meta) - Delta2t(t,meta) )/(s-sbplus);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}


complex<double>	ThreebodyAmp::Delta00B(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( Delta00(s,t,meta) - Delta1t(t,meta) )/(s-sbminus);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta00C(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = Delta00(s,t,meta)/(s-sbplus)/(s-sbminus)
        + Delta1t(t,meta)/(s-sbminus)/(4.0*meta*mpi)
        - Delta2t(t,meta)/(s-sbplus)/(4.0*meta*mpi);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}


complex<double>	ThreebodyAmp::Delta20A(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( 1.0+(pow(meta+mpi,2)-4.0*mpi*mpi)/(s-sbplus)+sbminus/(s-sbminus)
                                    +sbminus*(pow(meta+mpi,2)-4.0*mpi*mpi)/(s-sbminus)/(s-sbplus) )*Delta00(s,t,meta)
        +  sbminus*(pow(meta-mpi,2)-4.0*mpi*mpi)/(s-sbminus)/(4.0*meta*mpi)*Delta1t(t,meta)
        -  sbplus*(pow(meta+mpi,2)-4.0*mpi*mpi)/(s-sbplus)/(4.0*meta*mpi)*Delta2t(t,meta) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::Delta20B(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( Delta00(s,t,meta)-Delta2t(t,meta) )/(s-sbplus)/(s-sbplus) - Delta5t(t,meta)/(s-sbplus) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}






complex<double>	ThreebodyAmp::Delta20C(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( Delta00(s,t,meta)-Delta1t(t,meta) )/(s-sbminus)/(s-sbminus) - Delta4t(t,meta)/(s-sbminus) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::Delta20D(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  Delta20B(s,t,meta)/(s-sbminus) - ((Delta1t(t,meta)-Delta2t(t,meta))/(16.0*meta*meta*mpi*mpi)+Delta5t(t,meta)/(4.0*meta*mpi))/(s-sbminus) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}






complex<double>	ThreebodyAmp::Delta20E(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  Delta20C(s,t,meta)/(s-sbplus) - ((Delta2t(t,meta)-Delta1t(t,meta))/(16.0*meta*meta*mpi*mpi)-Delta4t(t,meta)/(4.0*meta*mpi))/(s-sbplus) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta20F(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  ( Delta20D(s,t,meta)  - Delta20E(s,t,meta) )/4.0/meta/mpi  ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::Delta20(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double ksq = s*(s-4.0*mpi*mpi)*(s-sbminus)*(s-sbplus)/s/s;
        
        complex<double> delta   = 3.0/2.0*Delta00(s,t,meta)
        + 3.0/2.0*2.0*(2.0*t+2.0*mpi*(meta-mpi))*Delta00A(s,t,meta)
        + 3.0/2.0*2.0*pow(meta-mpi,2)*Delta00B(s,t,meta)
        + 3.0/2.0*pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta20B(s,t,meta)
        + 3.0/2.0*4.0*pow(meta-mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))*Delta00C(s,t,meta)
        + 3.0/2.0*2.0*pow(meta-mpi,2)*pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta20D(s,t,meta)
        + 3.0/2.0*pow(meta-mpi,4)*( Delta20C(s,t,meta) + 2.0*(2.0*t+2.0*mpi*(meta-mpi))*Delta20E(s,t,meta) + pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta20F(s,t,meta) )
        - 1.0/2.0*Delta20A(s,t,meta);
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}









// ----------- new barrier ----------- //

complex<double>	ThreebodyAmp::NewDelta20A(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  (1.0-4.0*mpi*mpi/s)*Delta00C(s,t,meta) + 4.0*mpi*mpi/s*Delta00C(1.0e-20,t,meta) ;
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::NewDelta20(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double ksq = s*(s-4.0*mpi*mpi)*(s-sbminus)*(s-sbplus)/s/s;
        
        complex<double> delta   =  3.0/2.0*( Delta20C(s,t,meta) + 2.0*(2.0*t+2.0*mpi*(meta-mpi))*Delta20E(s,t,meta) + pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta20F(s,t,meta) )
        -  1.0/2.0*NewDelta20A(s,t,meta);
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}
// ------------------------- //










complex<double>	ThreebodyAmp::Delta21A(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  Delta0t(t,meta)
        + (pow(meta+mpi,2)-4.0*mpi*mpi)*Delta2t(t,meta)
        + pow(meta-mpi,2)*Delta1t(t,meta)
        + pow(meta-mpi,2)*(pow(meta+mpi,2)-4.0*mpi*mpi)/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta)) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::Delta21B(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = Delta0t(t,meta)
        + 2.0*(2.0*t+2.0*mpi*(meta-mpi))*Delta2t(t,meta)
        + 2.0*pow(meta-mpi,2)*Delta1t(t,meta)
        + pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta5t(t,meta)
        + pow(meta-mpi,4)*Delta4t(t,meta)
        + 4.0*pow(meta-mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))/(4.0*meta*mpi)*( Delta2t(t,meta) -Delta1t(t,meta) )
        + 2.0*pow(meta-mpi,2)*pow(2.0*t+2.0*mpi*(meta-mpi),2)/(4.0*meta*mpi)*( Delta5t(t,meta) -( Delta2t(t,meta) -Delta1t(t,meta) )/(4.0*meta*mpi))
        + 2.0*pow(meta-mpi,4)*(2.0*t+2.0*mpi*(meta-mpi))/(4.0*meta*mpi)*( -Delta4t(t,meta) + ( Delta2t(t,meta) -Delta1t(t,meta) )/(4.0*meta*mpi))
        + pow(meta-mpi,4)*pow(2.0*t+2.0*mpi*(meta-mpi),2)/pow(4.0*meta*mpi,2)*( Delta4t(t,meta) + Delta5t(t,meta) - ( Delta2t(t,meta) -Delta1t(t,meta) )/(2.0*meta*mpi)) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}


complex<double>	ThreebodyAmp::Delta21(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  3.0*Delta21B(t,meta) -  Delta21A(t,meta) + (2.0*s+t-meta*meta-3.0*mpi*mpi)*Delta20(s,t,meta)   ;
        
        return 3.0*delta;
    }
	else
    {
        return 0.0;
    }
}








// ----------- new barrier ----------- //

complex<double>	ThreebodyAmp::NewDelta21A(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  1.0/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta)) - 4.0*mpi*mpi*Delta00C(1.0e-20,t,meta) ;
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::NewDelta21B(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = Delta4t(t,meta)
        + 2.0*(2.0*t+2.0*mpi*(meta-mpi))/(4.0*meta*mpi)*( -Delta4t(t,meta) + ( Delta2t(t,meta) -Delta1t(t,meta) )/(4.0*meta*mpi))
        + pow(2.0*t+2.0*mpi*(meta-mpi),2)/pow(4.0*meta*mpi,2)*( Delta4t(t,meta) + Delta5t(t,meta) - ( Delta2t(t,meta) -Delta1t(t,meta) )/(2.0*meta*mpi)) ;
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}


complex<double>	ThreebodyAmp::NewDelta21(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  3.0*NewDelta21B(t,meta) -  NewDelta21A(t,meta) + (2.0*s+t-meta*meta-3.0*mpi*mpi)*NewDelta20(s,t,meta)   ;
        
        return t*3.0*delta;
    }
    else
    {
        return 0.0;
    }
}
// -------------------------- //







complex<double>	ThreebodyAmp::Delta22A(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  Delta3st(s,t,meta)
        + 3.0*(2.0*t+2.0*mpi*(meta-mpi))*(Delta0t(t,meta)-(s-sbplus)*Delta2t(t,meta))
        + 3.0/2.0*pow(2.0*t+2.0*mpi*(meta-mpi),2)*(Delta2t(t,meta)-(s-sbplus)*Delta5t(t,meta))
        + 3.0*pow(meta-mpi,2)*(Delta0t(t,meta)-(s-sbminus)*Delta1t(t,meta))
        + 3.0/2.0*4.0*pow(meta-mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))*(Delta2t(t,meta)-(s-sbminus)/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta)))
        + 3.0/2.0*2.0*pow(meta-mpi,2)*pow(2.0*t+2.0*mpi*(meta-mpi),2)*(Delta5t(t,meta)-(s-sbminus)/(4.0*meta*mpi)*( Delta5t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi)))
        + 3.0/2.0*pow(meta-mpi,4)*(Delta1t(t,meta)-(s-sbminus)*Delta4t(t,meta))
        + 3.0/2.0*2.0*pow(meta-mpi,4)*(2.0*t+2.0*mpi*(meta-mpi))*(Delta4t(t,meta)-(s-sbplus)/(4.0*meta*mpi)*( - Delta4t(t,meta) + (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi)))
        + 3.0/2.0*pow(meta-mpi,4)*pow(2.0*t+2.0*mpi*(meta-mpi),2)*( 1.0/(4.0*meta*mpi)*( Delta5t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi)) - (s-sbminus)/pow(4.0*meta*mpi,2)*( Delta5t(t,meta) + Delta4t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(2.0*meta*mpi) ) )
        - 1.0/2.0*( pow(meta+mpi,2) -4.0*mpi*mpi )*(  Delta0t(t,meta) - (s-sbplus)*Delta2t(t,meta) )
        - 1.0/2.0*pow(meta-mpi,2)*(  Delta0t(t,meta) - (s-sbminus)*Delta1t(t,meta) )
        - 1.0/2.0*pow(meta-mpi,2)*( pow(meta+mpi,2) -4.0*mpi*mpi )*( Delta2t(t,meta)-(s-sbminus)/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta))  ) ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta22B(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   =  Delta0t(t,meta)
        + 3.0*(2.0*t+2.0*mpi*(meta-mpi))*Delta2t(t,meta)
        + 3.0/2.0*pow(2.0*t+2.0*mpi*(meta-mpi),2)*Delta5t(t,meta)
        + 3.0*pow(meta-mpi,2)*Delta1t(t,meta)
        + 3.0/2.0*4.0*pow(meta-mpi,2)*(2.0*t+2.0*mpi*(meta-mpi))*(1.0/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta)))
        + 3.0/2.0*2.0*pow(meta-mpi,2)*pow(2.0*t+2.0*mpi*(meta-mpi),2)*(1.0/(4.0*meta*mpi))*(Delta5t(t,meta)-  (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi))
        + 3.0/2.0*pow(meta-mpi,4)*Delta4t(t,meta)
        + 3.0/2.0*2.0*pow(meta-mpi,4)*(2.0*t+2.0*mpi*(meta-mpi))*(1.0/(4.0*meta*mpi))*( - Delta4t(t,meta) + (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi))
        + 3.0/2.0*pow(meta-mpi,4)*pow(2.0*t+2.0*mpi*(meta-mpi),2)*(1.0/pow(4.0*meta*mpi,2))*(   Delta5t(t,meta) + Delta4t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(2.0*meta*mpi) )
        - 1.0/2.0*( pow(meta+mpi,2) -4.0*mpi*mpi )*Delta2t(t,meta)
        - 1.0/2.0*pow(meta-mpi,2)*Delta1t(t,meta)
        - 1.0/2.0*pow(meta-mpi,2)*( pow(meta+mpi,2) -4.0*mpi*mpi )*1.0/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta))   ;
        
        return delta;
    }
	else
    {
        return 0.0;
    }
}



complex<double>	ThreebodyAmp::Delta22(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double ktsq = t*(t-4.0*mpi*mpi)*(t-sbminus)*(t-sbplus)/t/t;
        
        complex<double> delta   = 6.0*Delta22A(s,t,meta)
                                + 6.0*(2.0*s+t-meta*meta-3.0*mpi*mpi)*Delta22B(t,meta)
                                + (3.0*pow(2.0*s+t-meta*meta-3.0*mpi*mpi,2) - ktsq)/2.0*Delta20(s,t,meta)     ;
        
        return 5.0*delta;
    }
	else
    {
        return 0.0;
    }
}









// ----------- new barrier ----------- //

complex<double>	ThreebodyAmp::NewDelta22A(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = 3.0/2.0*(Delta1t(t,meta)-(s-sbminus)*Delta4t(t,meta))
        + 3.0/2.0*2.0*(2.0*t+2.0*mpi*(meta-mpi))*(Delta4t(t,meta)-(s-sbplus)/(4.0*meta*mpi)*( - Delta4t(t,meta) + (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi)))
        + 3.0/2.0*pow(2.0*t+2.0*mpi*(meta-mpi),2)*( 1.0/(4.0*meta*mpi)*( Delta5t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi)) - (s-sbminus)/pow(4.0*meta*mpi,2)*( Delta5t(t,meta) + Delta4t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(2.0*meta*mpi) ) )
        
        
        - 1.0/2.0*( Delta2t(t,meta)-(s-sbminus)/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta))  )
        + 4.0*mpi*mpi/2.0/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta)) - 4.0*mpi*mpi/2.0*s*Delta00C(1.0e-20,t,meta)   ;
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::NewDelta22B(double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        complex<double> delta   = 3.0/2.0*Delta4t(t,meta)
        + 3.0/2.0*2.0*(2.0*t+2.0*mpi*(meta-mpi))*(1.0/(4.0*meta*mpi))*( - Delta4t(t,meta) + (Delta2t(t,meta)-Delta1t(t,meta))/(4.0*meta*mpi))
        + 3.0/2.0*pow(2.0*t+2.0*mpi*(meta-mpi),2)*(1.0/pow(4.0*meta*mpi,2))*(   Delta5t(t,meta) + Delta4t(t,meta) - (Delta2t(t,meta)-Delta1t(t,meta))/(2.0*meta*mpi) )
        
        - 1.0/2.0*1.0/(4.0*meta*mpi)*(Delta2t(t,meta)-Delta1t(t,meta))
        + 4.0*mpi*mpi/2.0*Delta00C(1.0e-20,t,meta);
        
        return delta;
    }
    else
    {
        return 0.0;
    }
}




complex<double>	ThreebodyAmp::NewDelta22(double s,double t, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
    if ((t<pow(meta-mpi,2))&&(t>0.0))
    {
        double ktsq = t*(t-4.0*mpi*mpi)*(t-sbminus)*(t-sbplus)/t/t;
        
        complex<double> delta   =  6.0*NewDelta22A(s,t,meta) + 6.0*(2.0*s+t-meta*meta-3.0*mpi*mpi)*NewDelta22B(t,meta) + (3.0*pow(2.0*s+t-meta*meta-3.0*mpi*mpi,2) - ktsq)/2.0*NewDelta20(s,t,meta)     ;
        
        return t*t*5.0*delta;
    }
    else
    {
        return 0.0;
    }
}



// ----------------------------- //





// -------------------- //





complex<double>	ThreebodyAmp::DeltaL1L2(int L1, int L2, double s,double t, double meta) const
{
	complex<double> delta (0.0,0.0);
    
    if ((L1==0)&&(L2==0))
    {
        delta = Delta00(s,t,meta);
    }
    else if ((L1==0)&&(L2==1))
    {
        delta = Delta01(s,t,meta);
    }
    else if ((L1==0)&&(L2==2))
    {
        delta = Delta02(s,t,meta);
    }
    else if ((L1==1)&&(L2==0))
    {
        delta = Delta10(s,t,meta);
    }
    else if ((L1==1)&&(L2==1))
    {
        delta = Delta11(s,t,meta);
    }
    else if ((L1==1)&&(L2==2))
    {
        delta = Delta12(s,t,meta);
    }
    else if ((L1==2)&&(L2==0))
    {
        delta = Delta20(s,t,meta);
    }
    else if ((L1==2)&&(L2==1))
    {
        delta = Delta21(s,t,meta);
    }
    else if ((L1==2)&&(L2==2))
    {
        delta = Delta22(s,t,meta);
    }
    else
    {
        delta = 0.0;
    }
    return delta;
}








complex<double>	ThreebodyAmp::NewDeltaL1L2(int L1, int L2, double s,double t, double meta) const
{
    complex<double> delta (0.0,0.0);
    
    if ((L1==0)&&(L2==0))
    {
        delta = Delta00(s,t,meta);
    }
    else if ((L1==0)&&(L2==1))
    {
        delta = t*Delta01(s,t,meta);
    }
    else if ((L1==0)&&(L2==2))
    {
        delta = t*t*Delta02(s,t,meta);
    }
    else if ((L1==1)&&(L2==0))
    {
        delta = (Delta10(s,t,meta) - Delta10(1.0e-20,t,meta))/s;
    }
    else if ((L1==1)&&(L2==1))
    {
        delta = t*(Delta11(s,t,meta) - Delta11(1.0e-20,t,meta))/s;
    }
    else if ((L1==1)&&(L2==2))
    {
        delta = t*t*(Delta12(s,t,meta) - Delta12(1.0e-20,t,meta))/s;
    }
    else if ((L1==2)&&(L2==0))
    {
        delta = NewDelta20(s,t,meta);
    }
    else if ((L1==2)&&(L2==1))
    {
        delta = NewDelta21(s,t,meta);
    }
    else if ((L1==2)&&(L2==2))
    {
        delta = NewDelta22(s,t,meta);
    }
    
    
    else
    {
        delta = 0.0;
    }
    return delta;
}






complex<double>	ThreebodyAmp::AdlerZeroDeltaL1L2(int L1, int L2, double s,double t, double meta, int I1) const
{
    complex<double> delta (0.0,0.0);
    
    if (L1==0)
    {
        if (I1==0)
        {
            delta = NewDeltaL1L2(0,L2,s,t,meta)  + (sAD_0_0-sAD_0_A)/(s-sAD_0_0)*( NewDeltaL1L2(0,L2,s,t,meta)-NewDeltaL1L2(0,L2,sAD_0_0,t,meta) ) ;
        }
        else if (I1==2)
        {
            delta = NewDeltaL1L2(0,L2,s,t,meta)  + (sAD_2_0-sAD_2_A)/(s-sAD_2_0)*( NewDeltaL1L2(0,L2,s,t,meta)-NewDeltaL1L2(0,L2,sAD_2_0,t,meta) ) ;
        }
        else if (I1==1)
        {
            delta = 0.0;
        }
        
    }
    else
    {
        delta = NewDeltaL1L2(L1,L2,s,t,meta);
    }
    
    return delta;
}








complex<double>	ThreebodyAmp::Ubelow(double s, double meta) const
{
    double sbminus = pow(meta-mpi,2);
    double sbplus  = pow(meta+mpi,2);
    
	complex<double>	 U;
    
	if ((s>=4.0*mpi*mpi)&&(s<sbminus))
	{
		U = sqrt(s*(s-4.0*mpi*mpi)*(s-sbminus)*(s-sbplus));
	}
	else if ((s>sbminus)&&(s<sbplus))
	{
		U = aIm*sqrt(-s*(s-4.0*mpi*mpi)*(s-sbminus)*(s-sbplus));
	}
	else if (s>sbplus)
	{
		U = - sqrt(s*(s-4.0*mpi*mpi)*(s-sbminus)*(s-sbplus));
	}
	else
	{
		U = 0.0;
	}
	
	return U;
}




complex<double>	ThreebodyAmp::Splus(double s, double meta) const
{
	complex<double> amp = ( -s + meta*meta + 3.0*mpi*mpi)/2.0 + Ubelow(s,meta)/2.0/s;
    
	return amp;
}


complex<double>	ThreebodyAmp::Sminus(double s, double meta) const
{
	complex<double> amp = ( -s + meta*meta + 3.0*mpi*mpi)/2.0 - Ubelow(s,meta)/2.0/s;
    
	return amp;
}













// computing parameter space for given process //

void	ThreebodyAmp::parameterspace3b(int m1, int m2, int m3, int Imax, int Lmax)
{
    paramfitnum3b = 0;
    
    cout << "****************************************************" << endl;
    cout << " Determing fitting parameters space a[I][L] for process: " << endl;
    cout << " eta -> pi[" << m1 << "]+pi["<< m2 << "]+pi["<< m3 << "]." << endl;
   
    cout << " non-zero a[I][L] are: " << endl;
    cout << endl;
    
    for (int Is=0; Is<=Imax; Is++)
    {
        for (int L=0; L<=Lmax; L++)
        {
            if ((int(1.0+pow(-1.0,Is+L))!=0)&&((IS_coef(Is,m1,m2,m3)!=0.0)||(IS_coef(Is,m2,m3,m1)!=0.0)||(IS_coef(Is,m3,m1,m2)!=0.0)))
            {
                cout << " a["<<Is<<"]["<< L << "] " << endl;
                paramfitnum3b = paramfitnum3b + 1 ;
                a0fit3b_I[paramfitnum3b] = Is;
                a0fit3b_L[paramfitnum3b] = L;
            }
        }
    }
    cout << endl;
    
    cout << "************************************************" << endl;
}

// ========================================== //







// initialization of fitting parameters //


void	ThreebodyAmp::parameters_ini_3b(string inputfile, int m1, int m2, int m3, int Imax, int Lmax,  struct param3b_space *pa0_IL)
{
    ifstream inpars(inputfile.c_str());
    if (inpars.fail())
    {
        cout  << "Error Opening File: " << inputfile << endl;
        exit(1);
    }
    
    
	for (int Is=0; Is<=2; Is++)
	{
        for (int L=0; L<=2; L++)
        {
            string paranames;
            inpars >> paranames >> (*pa0_IL).a0_I_L_struct[Is][L];
        }
	}
	inpars.close();
    
}
// ==================================== //

























// fitting part of G function around x=y=0 //




void ThreebodyAmp::fitG(int Is, int Ls, int It, int Lt, int nreim)
{
    
    
    int ncount = 0;
    double xfit[1000];
    double yfit[1000];
    for (int i=1; i<=ndimright; i++)
    {
        if ((svec_struct[i]<sgfitup)&&(svec_struct[i]>sgfitlow)) // dx~0.1 //
        {
            xfit[ncount] = svec_struct[i];
            if (nreim==0)
            {
                yfit[ncount] = GLvecre_struct[Is][Ls][It][Lt][i];
            }
            else if (nreim==1)
            {
                yfit[ncount] = GLvecim_struct[Is][Ls][It][Lt][i];
            }
        
            
            ncount = ncount + 1;
        }
    }
    ncount = ncount - 2;
    
    
    
    double  chisq;
    
     gsl_matrix *X, *cov;
     gsl_vector *y, *w, *c;
     
     X = gsl_matrix_alloc (ncount, fitdim);
     y = gsl_vector_alloc (ncount);
     w = gsl_vector_alloc (ncount);
     
     c = gsl_vector_alloc (fitdim);
     cov = gsl_matrix_alloc (fitdim, fitdim);
    
    
    for (int i = 0; i < ncount; i++)
    {
        for (int ii=0; ii<fitdim; ii++)
        {
            gsl_matrix_set (X, i, ii, pow(xfit[i],ii));
        }
    
        gsl_vector_set (y, i, yfit[i]);
        gsl_vector_set (w, i, 1.0/(yfit[i]*yfit[i]*0.0000001));
    }
    
    gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (ncount, fitdim);
    gsl_multifit_wlinear (X, w, y, c, cov, &chisq, work);
    gsl_multifit_linear_free (work);
    
    
    
    
    for (int i=0; i<fitdim; i++)
    {
        if (nreim==0)
        {
            grefitparam[Is][Ls][It][Lt][i] = gsl_vector_get(c,i);
        }
        else if (nreim==1)
        {
            gimfitparam[Is][Ls][It][Lt][i] = gsl_vector_get(c,i);
        }
    }
  
    
    gsl_matrix_free (X);
    gsl_vector_free (y);
    gsl_vector_free (w);
    gsl_vector_free (c);
    gsl_matrix_free (cov);
    
}






void ThreebodyAmp::fitGit(int m1, int m2, int m3, int Imax, int Lmax)
{
    
    // determine the size of matrix //
 
    for (int Is=0; Is<=Imax; Is++)
    {
        for (int Ls=0; Ls<=Lmax; Ls++)
        {
            if ((int(1.0+pow(-1.0,Is+Ls))!=0)&&((IS_coef(Is,m1,m2,m3)!=0.0)||(IS_coef(Is,m2,m3,m1)!=0.0)||(IS_coef(Is,m3,m1,m2)!=0.0)))
            {
                
                for (int It=0; It<=Imax; It++)
                {
                    for (int Lt=0; Lt<=Lmax; Lt++)
                    {
                        if ((int(1.0+pow(-1.0,It+Lt))!=0)&&((IS_coef(It,m1,m2,m3)!=0.0)||(IS_coef(It,m2,m3,m1)!=0.0)||(IS_coef(It,m3,m1,m2)!=0.0)))
                        {
                            for (int ii=0; ii<=1; ii++)
                            {
                                fitG(Is,Ls,It,Lt,ii);
                            }
                        }
                    }
                }    // ---------------------------- //
                
            }
        }
    }    // ---------------------------- //
  
    
}





complex<double> ThreebodyAmp::GILfit(int Is, int Ls, int It, int Lt, double s) const
{
    double gre = 0.0;
    double gim = 0.0;
    for (int i=0; i<fitdim; i++)
    {
        gre = gre + grefitparam[Is][Ls][It][Lt][i]*pow(s,i);
        gim = gim + gimfitparam[Is][Ls][It][Lt][i]*pow(s,i);
    }
    
    complex<double> gIL (gre,gim);
    
    return gIL;
}























// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //
