// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //

#include <iostream>
#include <string>
#include <complex>
#include "cvert_and_basicfuncs.h"


using namespace std;











// Isospin coefficients //

double BasicFuncs::IS_coef(int t, int m1, int m2, int m3) const
{
    double r = 0.0;
    for (int m = -1; m<=1; m++)
    {
        r = r + pow(-1.0,m3)*CG_coef(1,1,t,m1,m2,m)*CG_coef(1,1,t,-m3,0,m);
    }
    
    return r;
}

// --------------- //



// CG coefficients //

double BasicFuncs::CG_coef(int j1, int j2, int j3, int m1, int m2, int m3) const
{
    double r = pow(-1.0,-j1+j2-m3)*sqrt(2.0*j3+1.0)*gsl_sf_coupling_3j(2*j1,2*j2,2*j3,2*m1,2*m2,-2*m3);
    
    return r;
}

// --------------- //






double	BasicFuncs::Ksq(double s, double meta,double mass1, double mass2, double mass3) const
{
	double	 U = (s-pow(mass1-mass2,2))*(s-pow(mass1+mass2,2))*(s-pow(meta-mass3,2))*(s-pow(meta+mass3,2))/s/s ;
	
	return U;
}






// complex square root.

complex<double>	BasicFuncs::cpsqrt(complex<double> tc) const
{
	complex<double> cpsq (0.0,0.0);
	
	double retc = real(tc);
	double imtc = imag(tc);
	
	if (retc>=0.0)
	{
		if (imtc>=0.0)
		{
			cpsq = sqrt(tc);
		}
		else
		{
			cpsq = - sqrt(tc);
		}
		
	}
	else
	{
		if (imtc>=0.0)
	    {
			cpsq = sqrt(tc);
	    }
		else
		{
			cpsq = - sqrt(tc);
		}
		
	}
	
	
	
	return cpsq;
}











// ============== Begining of GaussLegendre Point and Weight =====================

void BasicFuncs::gaulegf(double x1, double x2, double x[], double w[], int n) const
{
	int i, j, m;
	double eps = 3.0E-14;
	double p1, p2, p3, pp, xl, xm, z, z1;
	
	m = (n+1)/2;
	xm = 0.5*(x2+x1);
	xl = 0.5*(x2-x1);
	for(i=1; i<=m; i++)
	{
		z = cos(3.141592654*((double)i-0.25)/((double)n+0.5));
		while(1)
		{
			p1 = 1.0;
			p2 = 0.0;
			for(j=1; j<=n; j++)
			{
				p3 = p2;
				p2 = p1;
				p1 = ((2.0*(double)j-1.0)*z*p2-((double)j-1.0)*p3)/
				(double)j;
			}
			pp = (double)n*(z*p1-p2)/(z*z-1.0);
			z1 = z;
			z = z1 - p1/pp;
			if(abs(z-z1) <= eps) break;
		}
		x[i] = xm - xl*z;
		x[n+1-i] = xm + xl*z;
		w[i] = 2.0*xl/((1.0-z*z)*pp*pp);
		w[n+1-i] = w[i];
	}
} /* end gaulegf */









// *****************************************

double BasicFuncs::enatk(double *x,double *y,int n,double t, double eps) const

{ int i,j,k,m,l;
    double z, xx[10], yy[10];
    
    z=0.0;
    if (n<1) return(z);
    if (n==1) { z=y[0]; return(z);}
    m=10;
    if (m>n) m=n;
	if (t<=x[0]) k=1;
	else if (t>=x[n-1]) k=n;
	else
	{ k=1; j=n;
		while ((k-j!=1)&&(k-j!=-1))
		{ l=(k+j)/2;
			if (t<x[l-1]) j=l;
			else k=l;
		}
		if (fabs(t-x[l-1])>fabs(t-x[j-1])) k=j;
	}
    j=1; l=0;
    for (i=1;i<=m;i++)
	{ k=k+j*l;
        if ((k<1)||(k>n))
		{ l=l+1; j=-j; k=k+j*l;}
        xx[i-1]=x[k-1]; yy[i-1]=y[k-1];
        l=l+1; j=-j;
	}
    i=0;
    do
	{ i=i+1; z=yy[i];
        for (j=0;j<=i-1;j++)
			z=yy[j]+(t-xx[j])*(yy[j]-z)/(xx[j]-xx[i]);
        yy[i]=z;
	}
    while ((i!=m-1)&&(fabs(yy[i]-yy[i-1])>eps));
    return(z);
	
}





double BasicFuncs::enlg3(double *x,double *y,int n,double t) const
{ int i,j,k,m;
    double z,s;
    z=0.0;
    if (n<1) return(z);
    if (n==1) { z=y[0]; return(z);}
    if (n==2)
	{ z=(y[0]*(t-x[1])-y[1]*(t-x[0]))/(x[0]-x[1]);
        return(z);
	}
    if (t<=x[1]) { k=0; m=2;}
    else if (t>=x[n-2]) { k=n-3; m=n-1;}
    else
	{ k=1; m=n;
        while (m-k!=1)
		{ i=(k+m)/2;
            if (t<x[i-1]) m=i;
            else k=i;
		}
        k=k-1; m=m-1;
        if (fabs(t-x[k])<fabs(t-x[m])) k=k-1;
        else m=m+1;
	}
    z=0.0;
    for (i=k;i<=m;i++)
	{ s=1.0;
        for (j=k;j<=m;j++)
			if (j!=i) s=s*(t-x[j])/(x[i]-x[j]);
        z=z+s*y[i];
	}
    return(z);
}









// matrix inversion subroutine//









//****************************************************************************80

void BasicFuncs::zgedi ( complex <double> a[], int lda, int n, int ipvt[],
                        complex <double> det[2], int job ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZGEDI computes the determinant and inverse of a matrix.
//
//  Discussion:
//
//    The matrix must have been factored by ZGECO or ZGEFA.
//
//    A division by zero will occur if the input factor contains
//    a zero on the diagonal and the inverse is requested.
//    It will not occur if the subroutines are called correctly
//    and if ZGECO has set 0.0 < RCOND or ZGEFA has set
//    INFO == 0.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    21 May 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch and Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, (Society for Industrial and Applied Mathematics),
//    3600 University City Science Center,
//    Philadelphia, PA, 19104-2688.
//
//  Parameters:
//
//    Input/output, complex <double> A[LDA*N]; on input, the factor information
//    from ZGECO or ZGEFA.  On output, the inverse matrix, if it
//    was requested,
//
//    Input, int LDA, the leading dimension of A.
//
//    Input, int N, the order of the matrix.
//
//    Input, int IPVT[N], the pivot vector from ZGECO or ZGEFA.
//
//    Output, complex <double> DET[2], the determinant of the original matrix,
//    if requested.  Otherwise not referenced.
//    Determinant = DET(1) * 10.0**DET(2) with
//    1.0 <= zabs1 ( DET(1) ) < 10.0 or DET(1) == 0.0.
//    Also, DET(2) is strictly real.
//
//    Input, int JOB.
//    11, both determinant and inverse.
//    01, inverse only.
//    10, determinant only.
//
{
    int i;
    int j;
    int k;
    int l;
    complex <double> t;
    complex <double> *work;
    //
    //  Compute the determinant.
    //
    if ( job / 10 != 0 )
    {
        det[0] = complex <double> ( 1.0, 0.0 );
        det[1] = complex <double> ( 0.0, 0.0 );
        
        for ( i = 1; i <= n; i++ )
        {
            if ( ipvt[i-1] != i )
            {
                det[0] = -det[0];
            }
            
            det[0] = a[i-1+(i-1)*lda] * det[0];
            
            if ( zabs1 ( det[0] ) == 0.0 )
            {
                break;
            }
            
            while ( zabs1 ( det[0] ) < 1.0 )
            {
                det[0] = det[0] * complex <double> ( 10.0, 0.0 );
                det[1] = det[1] - complex <double> ( 1.0, 0.0 );
            }
            
            while ( 10.0 <= zabs1 ( det[0] ) )
            {
                det[0] = det[0] / complex <double> ( 10.0, 0.0 );
                det[1] = det[1] + complex <double> ( 1.0, 0.0 );
            }
        }
    }
    //
    //  Compute inverse(U).
    //
    if ( ( job % 10 ) != 0 )
    {
        work = new complex <double>[n];
        
        for ( k = 1; k <= n; k++ )
        {
            a[k-1+(k-1)*lda] = complex <double> ( 1.0, 0.0 ) / a[k-1+(k-1)*lda];
            t = -a[k-1+(k-1)*lda];
            zscal ( k-1, t, a+0+(k-1)*lda, 1 );
            
            for ( j = k+1; j <= n; j++ )
            {
                t = a[k-1+(j-1)*lda];
                a[k-1+(j-1)*lda] = complex <double> ( 0.0, 0.0 );
                zaxpy ( k, t, a+0+(k-1)*lda, 1, a+0+(j-1)*lda, 1 );
            }
        }
        //
        //  Form inverse(U) * inverse(L).
        //
        for ( k = n-1; 1 <= k; k-- )
        {
            for ( i = k+1; i <= n; i++ )
            {
                work[i-1] = a[i-1+(k-1)*lda];
                a[i-1+(k-1)*lda] = complex <double> ( 0.0, 0.0 );
            }
            
            for ( j = k+1; j <= n; j++ )
            {
                t = work[j-1];
                zaxpy ( n, t, a+0+(j-1)*lda, 1, a+0+(k-1)*lda, 1 );
            }
            
            l = ipvt[k-1];
            
            if ( l != k )
            {
                zswap( n, a+0+(k-1)*lda, 1, a+0+(l-1)*lda, 1 );
            }
        }
        
        delete [] work;
    }
    
    return;
}
//****************************************************************************80


double BasicFuncs::zabs1 ( complex <double> z ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZABS1 returns the L1 norm of a number.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//    The L1 norm of a complex number is the sum of the absolute values
//    of the real and imaginary components.
//
//    ZABS1 ( Z ) = abs ( real ( Z ) ) + abs ( imaginary ( Z ) )
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for FORTRAN usage,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, pages 308-323, 1979.
//
//  Parameters:
//
//    Input, complex <double> Z, the number whose norm is desired.
//
//    Output, double ZABS1, the L1 norm of Z.
//
{
	double value;
	
	value = r8_abs ( real ( z ) ) + r8_abs ( imag ( z ) );
	
	return value;
}



double BasicFuncs::r8_abs ( double x ) const

//****************************************************************************80
//
//  Purpose:
//
//    R8_ABS returns the absolute value of a number.
//
//  Discussion:
//
//    This routine uses double precision arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    02 April 2005
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, the quantity whose absolute value is desired.
//
//    Output, double R8_ABS, the absolute value of X.
//
{
	double value;
	
	if ( 0.0 <= x )
	{
		value = x;
	}
	else
	{
		value = -x;
	}
	return value;
}
//****************************************************************************80




void BasicFuncs::zscal ( int n, complex <double> ca, complex <double> cx[], int incx ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZSCAL scales a vector by a constant.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt.
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for Fortran Usage,
//    Algorithm 539,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, September 1979, pages 308-323.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vector.
//
//    Input, complex <double> CA, the multiplier.
//
//    Input/output, complex <double> CX[], the vector to be scaled.
//
//    Input, int INCX, the increment between successive entries of CX.
//
{
	int i;
	
	if ( n <= 0 || incx <= 0 )
	{
		return;
	}
	
	if ( incx == 1 )
	{
		for ( i = 0; i < n; i++ )
		{
			cx[i] = ca * cx[i];
		}
	}
	else
	{
		for ( i = 0; i < n; i++ )
		{
			cx[i*incx] = ca * cx[i*incx];
		}
	}
	return;
}
//****************************************************************************80




void BasicFuncs::zaxpy ( int n, complex <double> ca, complex <double> cx[],
                        int incx, complex <double> cy[], int incy ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZAXPY computes a constant times a vector plus a vector.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for Fortran Usage,
//    Algorithm 539,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, September 1979, pages 308-323.
//
//  Parameters:
//
//    Input, int N, the number of elements in CX and CY.
//
//    Input, complex <double> CA, the multiplier of CX.
//
//    Input, complex <double> CX[], the first vector.
//
//    Input, int INCX, the increment between successive entries of CX.
//
//    Input/output, complex <double> CY[], the second vector.
//    On output, CY(*) has been replaced by CY(*) + CA * CX(*).
//
//    Input, int INCY, the increment between successive entries of CY.
//
{
	int i;
	int ix;
	int iy;
	
	if ( n <= 0 )
	{
		return;
	}
	
	if ( zabs1 ( ca ) == 0.0 )
	{
		return;
	}
	
	if ( incx != 1 || incy != 1 )
	{
		if ( 0 <= incx )
		{
			ix = 0;
		}
		else
		{
			ix = ( -n + 1 ) * incx;
		}
		
		if ( 0 <= incy )
		{
			iy = 0;
		}
		else
		{
			iy = ( -n + 1 ) * incy;
		}
		
		for ( i = 0; i < n; i++ )
		{
			cy[iy] = cy[iy] + ca * cx[ix];
			ix = ix + incx;
			iy = iy + incy;
		}
	}
	else
	{
		for ( i = 0; i < n; i++ )
		{
			cy[i] = cy[i] + ca * cx[i];
		}
		
	}
	
	return;
}
//****************************************************************************80



void BasicFuncs::zswap ( int n, complex <double> cx[], int incx, complex <double> cy[],
                        int incy ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZSWAP interchanges two vectors.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for Fortran Usage,
//    Algorithm 539,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, September 1979, pages 308-323.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vectors.
//
//    Input/output, complex <double> CX[], one of the vectors to swap.
//
//    Input, int INCX, the increment between successive entries of CX.
//
//    Input/output, complex <double> CY[], one of the vectors to swap.
//
//    Input, int INCY, the increment between successive elements of CY.
//
{
	complex <double> ctemp;
	int i;
	int ix;
	int iy;
	
	if ( n <= 0 )
	{
		return;
	}
	
	if ( incx == 1 && incy == 1 )
	{
		for ( i = 0; i < n; i++ )
		{
			ctemp = cx[i];
			cx[i] = cy[i];
			cy[i] = ctemp;
		}
	}
	else
	{
		if ( 0 <= incx )
		{
			ix = 0;
		}
		else
		{
			ix = ( -n + 1 ) * incx;
		}
		
		if ( 0 <= incy )
		{
			iy = 0;
		}
		else
		{
			iy = ( -n + 1 ) * incy;
		}
		
		for ( i = 0; i < n; i++ )
		{
			ctemp = cx[ix];
			cx[ix] = cy[iy];
			cy[iy] = ctemp;
			ix = ix + incx;
			iy = iy + incy;
		}
	}
	
	return;
}













double BasicFuncs::zgeco ( complex <double> a[], int lda, int n, int ipvt[] ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZGECO factors a complex matrix and estimates its condition.
//
//  Discussion:
//
//    If RCOND is not needed, ZGEFA is slightly faster.
//
//    To solve A*X = B, follow ZGECO by ZGESL.
//
//    To compute inverse(A)*C, follow ZGECO by ZGESL.
//
//    To compute determinant(A), follow ZGECO by ZGEDI.
//
////////////////////////////////////////////////////////
//    To compute inverse(A), follow ZGECO by ZGEDI.
///////////////////////////////////////////////////////

//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    21 May 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch and Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, (Society for Industrial and Applied Mathematics),
//    3600 University City Science Center,
//    Philadelphia, PA, 19104-2688.
//
//  Parameters:
//
//    Input/output, complex <double> A[LDA*N], on input, the matrix to be
//    factored.  On output, an upper triangular matrix and the multipliers
//    used to obtain it.  The factorization can be written A = L*U where
//    L is a product of permutation and unit lower triangular matrices
//    and U is upper triangular.
//
//    Input, int LDA, the leading dimension of A.
//
//    Input, int N, the order of the matrix.
//
//    Output, int IPVT[N], the pivot indices.
//
//    Output, double SGECO, an estimate of the reciprocal condition of A.
//    For the system A*X = B, relative perturbations in A and B of size
//    EPSILON may cause relative perturbations in X of size (EPSILON/RCOND).
//    If RCOND is so small that the logical expression
//      1.0 + RCOND == 1.0
//    is true, then A may be singular to working precision.  In particular,
//    RCOND is zero if exact singularity is detected or the estimate
//    underflows.
//
//  Local Parameters:
//
//    Workspace, complex <double> Z[N], a work vector whose contents are usually
//    unimportant.  If A is close to a singular matrix, then Z is
//    an approximate null vector in the sense that
//      norm ( A * Z ) = RCOND * norm ( A ) * norm ( Z ).
//
{
	double anorm;
	complex <double> ek;
	int i;
	int info;
	int j;
	int k;
	int l;
	double rcond;
	double s;
	double sm;
	complex <double> t;
	complex <double> wk;
	complex <double> wkm;
	double ynorm;
	complex <double> *z;
	
	z = new complex <double> [n];
	//
	//  Compute the 1-norm of A.
	//
	anorm = 0.0;
	for ( j = 0; j < n; j++ )
	{
		anorm = r8_max ( anorm, dzasum ( n, a+0+j*lda, 1 ) );
	}
	//
	//  Factor.
	//
	info = zgefa ( a, lda, n, ipvt );
	//
	//  RCOND = 1/(norm(A)*(estimate of norm(inverse(A)))).
	//
	//  Estimate = norm(Z)/norm(Y) where A*Z = Y and hermitian(A)*Y = E.
	//
	//  Hermitian(A) is the conjugate transpose of A.
	//
	//  The components of E are chosen to cause maximum local
	//  growth in the elements of W where hermitian(U)*W = E.
	//
	//  The vectors are frequently rescaled to avoid overflow.
	//
	//  Solve hermitian(U)*W = E.
	//
	ek = complex <double> ( 1.0, 0.0 );
	for ( i = 0; i < n; i++ )
	{
		z[i] = complex <double> ( 0.0, 0.0 );
	}
	
	for ( k = 1; k <= n; k++ )
	{
		if ( zabs1 ( z[k-1] ) != 0.0 )
		{
			ek = zsign1 ( ek, -z[k-1] );
		}
		
		if ( zabs1 ( a[k-1+(k-1)*lda] ) < zabs1 ( ek - z[k-1] ) )
		{
			s = zabs1 ( a[k-1+(k-1)*lda] ) / zabs1 ( ek - z[k-1] );
			zdscal ( n, s, z, 1 );
			ek = complex <double> ( s, 0.0 ) * ek;
		}
		
		wk = ek - z[k-1];
		wkm = -ek - z[k-1];
		s = zabs1 ( wk );
		sm = zabs1 ( wkm );
		
		if ( zabs1 ( a[k-1+(k-1)*lda] ) != 0.0 )
		{
			wk = wk / conj ( a[k-1+(k-1)*lda] );
			wkm = wkm / conj ( a[k-1+(k-1)*lda] );
		}
		else
		{
			wk = complex <double> ( 1.0, 0.0 );
			wkm = complex <double> ( 1.0, 0.0 );
		}
		
		for ( j = k+1; j <= n; j++ )
		{
			sm = sm + zabs1 ( z[j-1] + wkm * conj ( a[k-1+(j-1)*lda] ) );
			z[j-1] = z[j-1] + wk * conj ( a[k-1+(j-1)*lda] );
			s = s + zabs1 ( z[j-1] );
		}
		
		if ( s < sm )
		{
			t = wkm - wk;
			wk = wkm;
			for ( j = k+1; j <= n; j++ )
			{
				z[j-1] = z[j-1] + t * conj ( a[k-1+(j-1)*lda] );
			}
		}
		z[k-1] = wk;
	}
	
	s = 1.0 / dzasum ( n, z, 1 );
	zdscal ( n, s, z, 1 );
	//
	//  Solve hermitian(L) * Y = W.
	//
	for ( k = n; 1 <= k; k-- )
	{
		if ( k < n )
		{
			z[k-1] = z[k-1] + zdotc ( n-k, a+k+(k-1)*lda, 1, z+k, 1 );
		}
		
		if ( 1.0 < zabs1 ( z[k-1] ) )
		{
			s = 1.0 / zabs1 ( z[k-1] );
			zdscal ( n, s, z, 1 );
		}
		l = ipvt[k-1];
		
		t      = z[l-1];
		z[l-1] = z[k-1];
		z[k-1] = t;
	}
	
	s = 1.0 / dzasum ( n, z, 1 );
	zdscal ( n, s, z, 1 );
	
	ynorm = 1.0;
	//
	//  Solve L * V = Y.
	//
	for ( k = 1; k <= n; k++ )
	{
		l = ipvt[k-1];
		
		t      = z[l-1];
		z[l-1] = z[k-1];
		z[k-1] = t;
		
		if ( k < n )
		{
			zaxpy ( n-k, t, a+k+(k-1)*lda, 1, z+k, 1 );
		}
		
		if ( 1.0 < zabs1 ( z[k-1] ) )
		{
			s = 1.0 / zabs1 ( z[k-1] );
			zdscal ( n, s, z, 1 );
			ynorm = s * ynorm;
		}
	}
	
	s = 1.0 / dzasum ( n, z, 1 );
	zdscal ( n, s, z, 1 );
	ynorm = s * ynorm;
	//
	//  Solve U * Z = V.
	//
	for ( k = n; 1 <= k; k-- )
	{
		if ( zabs1 ( a[k-1+(k-1)*lda] ) < zabs1 ( z[k-1] ) )
		{
			s = zabs1 ( a[k-1+(k-1)*lda] ) / zabs1 ( z[k-1] );
			zdscal ( n, s, z, 1 );
			ynorm = s * ynorm;
		}
		
		if ( zabs1 ( a[k-1+(k-1)*lda] ) != 0.0 )
		{
			z[k-1] = z[k-1] / a[k-1+(k-1)*lda];
		}
		else
		{
			z[k-1] = complex <double> ( 1.0, 0.0 );
		}
		
		t = -z[k-1];
		zaxpy ( k-1, t, a+0+(k-1)*lda, 1, z, 1 );
	}
	//
	//  Make ZNORM = 1.
	//
	s = 1.0 / dzasum ( n, z, 1 );
	zdscal ( n, s, z, 1 );
	ynorm = s * ynorm;
	
	if ( anorm != 0.0 )
	{
		rcond = ynorm / anorm;
	}
	else
	{
		rcond = 0.0;
	}
	
	delete [] z;
	
	return rcond;
}
//****************************************************************************80



complex <double> BasicFuncs::zsign1 ( complex <double> z1, complex <double> z2 ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZSIGN1 is a transfer-of-sign function.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//    The L1 norm is used.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, complex <double> Z1, Z2, the arguments.
//
//    Output, complex <double> ZSIGN1,  a complex value, with the magnitude of
//    Z1, and the argument of Z2.
//
{
	complex <double> value;
	
	if ( zabs1 ( z2 ) == 0.0 )
	{
		value = complex <double> ( 0.0, 0.0 );
	}
	else
	{
		value = zabs1 ( z1 ) * ( z2 / zabs1 ( z2 ) );
	}
	
	return value;
}
//****************************************************************************80

void BasicFuncs::zdscal ( int n, double sa, complex <double> cx[], int incx ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZDSCAL scales a vector by a constant.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//    The scaling constant is double precision real.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for Fortran Usage,
//    Algorithm 539,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, September 1979, pages 308-323.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vector.
//
//    Input, double SA, the multiplier.
//
//    Input/output, complex <double> CX[], the vector to be scaled.
//
//    Input, int INCX, the increment between successive entries of
//    the vector CX.
//
{
	int i;
	
	if ( n <= 0 || incx <= 0 )
	{
		return;
	}
	
	if ( incx == 1 )
	{
		for ( i = 0; i < n; i++ )
		{
			cx[i] = sa * cx[i];
		}
	}
	else
	{
		for ( i = 0; i < n; i++ )
		{
			cx[i*incx] = sa * cx[i*incx];
		}
	}
	return;
}
//****************************************************************************80



complex <double> BasicFuncs::zdotc ( int n, complex <double> cx[], int incx,
                                    complex <double> cy[], int incy ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZDOTC forms the conjugated dot product of two vectors.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for Fortran Usage,
//    Algorithm 539,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, September 1979, pages 308-323.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vectors.
//
//    Input, complex <double> CX[], the first vector.
//
//    Input, int INCX, the increment between successive entries in CX.
//
//    Input, complex <double> CY[], the second vector.
//
//    Input, int INCY, the increment between successive entries in CY.
//
//    Output, complex <double> ZDOTC, the conjugated dot product of
//    the corresponding entries of CX and CY.
//
{
	int i;
	int ix;
	int iy;
	complex <double> value ( 0.0, 0.0 );
	
	if ( n <= 0 )
	{
		return value;
	}
	
	if ( incx == 1 && incy == 1 )
	{
		for ( i = 0; i < n; i++ )
		{
			value = value + conj ( cx[i] ) * cy[i];
		}
	}
	else
	{
		if ( 0 <= incx )
		{
			ix = 0;
		}
		else
		{
			ix = ( -n + 1 ) * incx;
		}
		
		if ( 0 <= incy )
		{
			iy = 0;
		}
		else
		{
			iy = ( -n + 1 ) * incy;
		}
		
		for ( i = 0; i < n; i++ )
		{
			value = value + conj ( cx[ix] ) * cy[iy];
			ix = ix + incx;
			iy = iy + incy;
		}
	}
	return value;
}
//****************************************************************************80


double BasicFuncs::dzasum ( int n, complex <double> x[], int incx ) const

//****************************************************************************80
//
//  Purpose:
//
//    DZASUM takes the sum of the absolute values of a vector.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for FORTRAN usage,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, pages 308-323, 1979.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vector.
//
//    Input, complex <double> X[], the vector.
//
//    Input, int INCX, the increment between successive entries of X.
//
//    Output, double DZASUM, the sum of the absolute values.
//
{
	int i;
	int ix;
	double value;
	
	value = 0.0;
	
	if ( n <= 0 || incx <= 0 )
	{
		return value;
	}
	
	if ( incx == 1 )
	{
		for ( i = 0; i < n; i++ )
		{
			value = value + r8_abs ( real ( x[i] ) )
			+ r8_abs ( imag ( x[i] ) );
		}
	}
	else
	{
		ix = 0;
		for ( i = 0; i < n; i++ )
		{
			value = value + r8_abs ( real ( x[ix] ) )
			+ r8_abs ( imag ( x[ix] ) );
			ix = ix + incx;
		}
	}
	return value;
}
//****************************************************************************80









int BasicFuncs::zgefa ( complex <double> a[], int lda, int n, int ipvt[] ) const

//****************************************************************************80
//
//  Purpose:
//
//    ZGEFA factors a complex matrix by Gaussian elimination.
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    21 May 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch and Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, (Society for Industrial and Applied Mathematics),
//    3600 University City Science Center,
//    Philadelphia, PA, 19104-2688.
//
//  Parameters:
//
//    Input/output, complex <double> A[LDA*N]; on input, the matrix to be factored.
//    On output, an upper triangular matrix and the multipliers which were
//    used to obtain it.  The factorization can be written A = L*U where
//    L is a product of permutation and unit lower triangular matrices and
//    U is upper triangular.
//
//    Input, int LDA, the leading dimension of A.
//
//    Input, int N, the order of the matrix.
//
//    Output, int IPVT[N], the pivot indices.
//
//    Output, int ZGEFA,
//    0, normal value.
//    K, if U(K,K) == 0.0.  This is not an error condition for this
//    subroutine, but it does indicate that ZGESL or ZGEDI will divide by zero
//    if called.  Use RCOND in ZGECO for a reliable indication of singularity.
//
{
	int info;
	int j;
	int k;
	int l;
	complex <double> t;
	//
	//  Gaussian elimination with partial pivoting.
	//
	info = 0;
	
	for ( k = 1; k <= n-1; k++ )
	{
		//
		//  Find L = pivot index.
		//
		l = izamax ( n-k+1, a+(k-1)+(k-1)*lda, 1 ) + k - 1;
		ipvt[k-1] = l;
		//
		//  Zero pivot implies this column already triangularized.
		//
		if ( zabs1 ( a[l-1+(k-1)*lda] ) == 0.0 )
		{
			info = k;
			continue;
		}
		//
		//  Interchange if necessary.
		//
		if ( l != k )
		{
			t                = a[l-1+(k-1)*lda];
			a[l-1+(k-1)*lda] = a[k-1+(k-1)*lda];
			a[k-1+(k-1)*lda] = t;
		}
		//
		//  Compute multipliers
		//
		t = - complex <double> ( 1.0, 0.0 ) / a[k-1+(k-1)*lda];
		zscal ( n-k, t, a+k+(k-1)*lda, 1 );
		//
		//  Row elimination with column indexing
		//
		for ( j = k+1; j <= n; j++ )
		{
			t = a[l-1+(j-1)*lda];
			if ( l != k )
			{
				a[l-1+(j-1)*lda] = a[k-1+(j-1)*lda];
				a[k-1+(j-1)*lda] = t;
			}
			zaxpy ( n-k, t, a+k+(k-1)*lda, 1, a+k+(j-1)*lda, 1 );
		}
		
	}
	
	ipvt[n-1] = n;
	
	if ( zabs1 ( a[n-1+(n-1)*lda] ) == 0.0 )
	{
		info = n;
	}
	
	return info;
}
//****************************************************************************80




int BasicFuncs::izamax ( int n, complex <double> x[], int incx ) const

//****************************************************************************80
//
//  Purpose:
//
//    IZAMAX indexes the vector element of maximum absolute value.
//
//  Discussion:
//
//    This routine uses double precision complex arithmetic.
//
//    WARNING: This index is a 1-based index, not a 0-based index!
//
//  Licensing:
//
//    This code is distributed under the GNU LGPL license.
//
//  Modified:
//
//    15 April 2006
//
//  Author:
//
//    C++ version by John Burkardt
//
//  Reference:
//
//    Jack Dongarra, Cleve Moler, Jim Bunch, Pete Stewart,
//    LINPACK User's Guide,
//    SIAM, 1979.
//
//    Charles Lawson, Richard Hanson, David Kincaid, Fred Krogh,
//    Basic Linear Algebra Subprograms for FORTRAN usage,
//    ACM Transactions on Mathematical Software,
//    Volume 5, Number 3, pages 308-323, 1979.
//
//  Parameters:
//
//    Input, int N, the number of entries in the vector.
//
//    Input, complex <double> X[], the vector.
//
//    Input, int INCX, the increment between successive entries of X.
//
//    Output, int IZAMAX, the index of the element of maximum
//    absolute value.
//
{
	int i;
	int ix;
	double smax;
	int value;
	
	value = 0;
	
	if ( n < 1 || incx  <=  0 )
	{
		return value;
	}
	
	value = 1;
	
	if ( n == 1 )
	{
		return value;
	}
	
	if ( incx != 1 )
	{
		ix = 0;
		smax = zabs1 ( x[0] );
		ix = ix + incx;
		
		for ( i = 1; i < n; i++ )
		{
			if ( smax < zabs1 ( x[ix] ) )
			{
				value = i + 1;
				smax = zabs1 ( x[ix] );
			}
			ix = ix + incx;
		}
	}
	else
	{
		smax = zabs1 ( x[0] );
		for ( i = 1; i < n; i++ )
		{
			if ( smax < zabs1 ( x[i] ) )
			{
				value = i + 1;
				smax = zabs1 ( x[i] );
			}
		}
	}
	
	return value;
}
//****************************************************************************80



double BasicFuncs::r8_max ( double x, double y ) const

//****************************************************************************80
//
//  Purpose:
//
//    R8_MAX returns the maximum of two numbers.
//
//  Discussion:
//
//    This routine uses double precision arithmetic.
//
//  Modified:
//
//    18 August 2004
//
//  Author:
//
//    John Burkardt
//
//  Parameters:
//
//    Input, double X, Y, the quantities to compare.
//
//    Output, double R8_MAX, the maximum of X and Y.
//
{
	double value;
	
	if ( y < x )
	{
		value = x;
	}
	else
	{
		value = y;
	}
	return value;
}
//****************************************************************************80




// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //

