// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //


#include <iostream>
#include <fstream>
#include <iomanip>
#include <complex>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>

#include "../amps_LIB/Amps/3bamp.h"





using namespace std;


int Imax, Lmax;


int main( int argc, char* argv[]  )
{
    int m1 = +1;
    int m2 = -1;
    int m3 = 0;
    
    int KTon = 1;
    string datafilename;

    
    if (argc != 4)
    {
        cout << " " << endl;
        cout << " Input Imax, Lmax. " << endl;
        cout << " Usage: ./3bamp <Imax>  <Lmax> <filename>. " << endl;
        cout << " " << endl;
        exit (1);
    }
    else
    {
        sscanf(argv[1],"%d",&Imax) ;
        sscanf(argv[2],"%d",&Lmax) ;
        datafilename = argv[3];
    }
    
    cout << endl;
    cout << "****************************************************" << endl;
    cout << "Inputs are: Imax=" << Imax<<", Lmax=" << Lmax << ", 3b="<<KTon<< ", process is: pi["<<m1<<"]+pi["<< m2<<"]+pi["<<m3<<"]." <<endl;
    cout << "****************************************************" << endl;
    cout << endl;
   
    

    
    
    // initializing //
    
    double meta = 0.549;
    double mpip = 0.13957;
    double mpim = 0.13957;
    double mpi0 = 0.13498;

    double mpi = 0.13804;
    
    
    ThreebodyAmp dalizplot;
    dalizplot.parameterspace3b(m1,m2,m3,Imax,Lmax);
    
    double slowlimit = 0.005;
    
    dalizplot.gaulegf(slowlimit, pow(meta-mpi,2)-0.00001, dalizplot.xradright, dalizplot.weradright, dalizplot.ndimright);
    
    for (int i=1; i<=dalizplot.paramfitnum3b; i++)
    {
        int It = dalizplot.a0fit3b_I[i];
        int Lt = dalizplot.a0fit3b_L[i];
        dalizplot.GhatLinv(Imax,Lmax,It,Lt, meta);
    }
    
    dalizplot.metafit = meta;
    dalizplot.sgfitup = pow(meta-0.138,2) - 0.01;
    dalizplot.sgfitlow = pow(meta-0.138,2) - 0.05;
    dalizplot.fitGit(m1,m2,m3,Imax,Lmax);
    
    
    string inputfile = "./input_params/input_fitting_params.txt";
    struct param3b_space a0_IL;
    struct param3b_space *pa0_IL = &a0_IL ;
    
    dalizplot.parameters_ini_3b(inputfile,m1,m2,m3,Imax,Lmax,pa0_IL);
    
    
    
    // computing amplitudes //
    ifstream readfilein(datafilename.c_str());
    if (readfilein.fail())
    {
        cout  << "Error Opening File: " << datafilename << endl;
        exit(1);
    }
    
    
 
    
    while (!readfilein.eof())
    {
        double  x,y;
        readfilein >> x >> y;
        
        
        if (readfilein.eof())
        {
            break;
        }
        
        complex<double>	 r = dalizplot.amp_xy_3b(x,y,m1,m2,m3,Imax,Lmax,meta,mpip,mpim,mpi0,pa0_IL);;
        
        cout << x << " " << y << " " << real(r) << " " << imag(r) << endl;
        
        //cout << x << " " << y << " " << real(conj(r)*r)  << endl;

    }
    
    
    
 
}









// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //


